@@ -12,7 +12,7 @@ from ptrace.error import PTRACE_ERRORS, writeError
 from ptrace.binding import HAS_PTRACE_SINGLESTEP
 from ptrace.disasm import HAS_DISASSEMBLER
 from ptrace.ctypes_tools import (truncateWord,
-    formatWordHex, formatAddress, formatAddressRange)
+    formatWordHex, formatAddress, formatAddressRange, word2bytes)
 from ptrace.process_tools import dumpProcessInfo
 from ptrace.tools import inverseDict
 from ptrace.func_call import FunctionCallOptions
@@ -22,6 +22,8 @@ from ptrace.terminal import enableEchoMode, terminalWidth
 from errno import ESRCH
 from ptrace.cpu_info import CPU_POWERPC
 from ptrace.debugger import ChildError
+from ptrace.debugger.memory_mapping import readProcessMappings
+
 import re
 try:
     # Use readline for better raw_input()
@@ -32,6 +34,8 @@ except ImportError:
 # Match a register name: $eax, $gp0, $orig_eax
 REGISTER_REGEX = re.compile(r"\$[a-z]+[a-z0-9_]+")
 
+#BYTES_REGEX = re.compile(r"""(?:'([^'\\]*)'|"([^"\\]*)")""")
+
 SIGNALS = inverseDict(SIGNAMES)   # name -> signum
 
 COMMANDS = (
@@ -52,8 +56,8 @@ COMMANDS = (
     ("hexdump", "dump memory as specified address or address range (hexdump <address> or hexdump <start> <stop>)"),
     ("where2", "display original code content (don't show effects of breakpoint on code)"),
     ("stack", "display stack content"),
-    ("backtrace", "dump the bakctrace"),
-    ("proc", "display process informations"),
+    ("backtrace", "dump the backtrace"),
+    ("proc", "display process information"),
     ("maps", "display memory mappings"),
 
     # breakpoints
@@ -66,12 +70,26 @@ COMMANDS = (
     ("proclist", "list of traced processes"),
     ("switch", "switch active process (switch or switch <pid>)"),
 
+    ("follow", r'''follow a term (eg. "follow '\x12\x14\x27\x13'")'''),
+    ("showfollow", 'show all "followed" terms'),
+    ("resetfollow", 'reset all "followed" terms'),
+    ("xray", 'show addresses of (and possible pointers to) "followed" terms'),
+
     # other
-    ("dbginfo", "informations about the debugger"),
+    ("dbginfo", "information about the debugger"),
     ("quit", "quit debugger"),
     ("help", "display this help"),
 )
 
+# finds possible pointer values in process memory space,
+# pointing to address
+def getPointers(process, address):
+    address = word2bytes(address)
+    procmaps = readProcessMappings(process)
+    for pm in procmaps:
+        for found in pm.search(address):
+            yield found
+
 class Gdb(Application):
     def __init__(self):
         Application.__init__(self)
@@ -84,7 +102,7 @@ class Gdb(Application):
 
         self.last_signal = {}
 
-        # We assume user wants all possible informations
+        # We assume user wants all possible information
         self.syscall_options = FunctionCallOptions(
             write_types=True,
             write_argname=True,
@@ -94,6 +112,8 @@ class Gdb(Application):
         # FIXME: Remove self.breaks!
         self.breaks = dict()
 
+        self.followterms = []
+
     def setupLog(self):
         self._setupLog(stdout)
 
@@ -205,6 +225,41 @@ class Gdb(Application):
             values.append(value)
         return values
 
+    def parseBytes(self, text):
+        # FIXME: Validate input
+#        if not BYTES_REGEX.match(text):
+#            raise ValueError('Follow text must be enclosed in quotes!')
+        value = eval(text)
+        if not isinstance(value, str):
+            raise TypeError("Input is not a bytes string!")
+        return value
+
+    def addFollowTerm(self, text):
+        # Allow terms of the form 'string', "string", '\x04', "\x01\x14"
+        term = self.parseBytes(text)
+        self.followterms.append(term)
+
+    def showFollowTerms(self):
+        print self.followterms
+
+    def _xray(self):
+        for term in self.followterms:
+            for process in self.debugger:
+                for procmap in readProcessMappings(process):
+                    for address in procmap.search(term):
+                        yield (process, procmap, address, term)
+
+    # displays the offsets of all terms found in the process memory mappings
+    # along with possible addresses of pointers pointing to these terms
+    def xray(self):
+        for process, procmap, address, term in self._xray():
+            pointers = " ".join(formatAddress(ptr_addr)
+                for ptr_addr in getPointers(process, address))
+            print "term[%s] pid[%i] %s %s pointers: %s" % (
+                repr(term), process.pid, procmap,
+                formatAddress(address),
+                pointers)
+
     def execute(self, command):
         errmsg = None
         if command == "cont":
@@ -257,6 +312,14 @@ class Gdb(Application):
             errmsg = self.signal(command[7:])
         elif command.startswith("print "):
             errmsg = self.print_(command[6:])
+        elif command.startswith("follow "):
+            errmsg = self.addFollowTerm(command[7:])
+        elif command == "showfollow":
+            self.showFollowTerms()
+        elif command == "resetfollow":
+            self.followterms = []
+        elif command == "xray":
+            self.xray()
         else:
             errmsg = "Unknown command: %r" % command
         if errmsg:
@@ -702,7 +765,13 @@ class Gdb(Application):
             ok = True
             for command in command_str.split(";"):
                 command = command.strip()
-                ok &= self.execute(command)
+                try:
+                    ok &= self.execute(command)
+                except Exception, err:
+                    print "Command error: %s" % err
+                    ok = False
+                if not ok:
+                    break
             if ok:
                 self.previous_command = command_str
         except KeyboardInterrupt:
