@@ -88,7 +88,7 @@ class PtraceProcess:
        - disassemble(): assembler code of the next instructions
        - disassembleOne(): assembler code of the next instruction
        - findStack(): get stack memory mapping
-       - getsiginfo(): get signal informations
+       - getsiginfo(): get signal information
        - getBacktrace(): get the current backtrace
 
      * set status
@@ -420,14 +420,14 @@ class PtraceProcess:
         try:
             name, shift, mask = CPU_SUB_REGISTERS[name]
         except KeyError:
-            mask = 0
             shift = 0
+            mask = None
         if name not in REGISTER_NAMES:
             raise ProcessError(self, "Unknown register: %r" % name)
         regs = self.getregs()
         value = getattr(regs, name)
+        value >>= shift
         if mask:
-            value >>= shift
             value &= mask
         return value
 
